// (C) Bryan P. Bergeron, for Nuts & Volts Magazine
// Accompanies article "Translational Reality", Nuts & Volts Magazine
// Thanks to article in Learning About Electronics at
// http://www.learningaboutelectronics.com/Articles/MCP4131-digital-potentiometer-circuit.php
// and the data sheet at
// http://www.learningaboutelectronics.com/Datasheets/MCP-digital-potentiometer-ICs-datasheet.pdf
// Toaster pot higher resistance => darker toast
// Assumes Arduino Uno
// Sparkfun Soil Moisture Sensor
// Digital Pot 128 step 4131-503 (50K full scale)
// 20K pot on toaster in series with 20K resistor on Arduino
// When toaster pot = 0 -> 5V (1023) to analog input
// When toaster pot = 20K -> 2.5V (512) to analog input

#include <SPI.h>            // for digital pot
#include <Servo.h>          // for water pump

#define STOP          1500  // pump speeds; adjust as needed
#define SLOW          1800
#define MED           1850
#define FAST          1900
#define FASTER        1950
#define FASTEST       2000
#define controlPin       7  // Digital out to motor controller
#define ToasterPin       2  // Analog input, from toaster pot
#define DRY = 15;           // Minimum soil moisture value
byte address = 0x00;        // Digital pot
int CS = 10;                // Digital pot
int sensorPin = A0;         // Moisture senor
int PotValue = 0;           // Digital pot value
const int sensorMin = 0;    // soil moisture sensor
const int sensorMax = 1024; // soil moisture sensor
const int ToasterMin = 512; // toaster pot
const int ToasterMax = 1000;// toaster pot

Servo servo;

void setup()
{
pinMode (CS, OUTPUT);
SPI.begin();
Serial.begin(9600);                   // For debugging
delay(5);                             // HB-25 initialization time (5ms)                         
pinMode(controlPin, OUTPUT);
digitalWrite(controlPin, LOW);        // Set control pin low on power up                   
servo.attach(controlPin, 800, 2200);  // Attach HB-25, set range 0.8 - 2.2 ms
servo.writeMicroseconds(STOP);        // Part of HB-25 initialization  
}

void loop()
{
//First set the digital pot value, based on the soil moisture senor reading
//Adjust mapping to account for soil differences
// Max value returned by soil sensor when moist
int TempValue = analogRead(sensorPin);  
Serial.print("TempValue: ");
Serial.println(TempValue);

// ****** Default mapping ******
int range = map(TempValue, sensorMin, sensorMax, 0, 52);
Serial.print("Soil sensor (0-52): ");
Serial.println(range);

PotValue = range;

// ****** Alternative mapping 1 ******
// Uncomment example to use
// example 1.  If moisture below threshold, then darkest toast

//if (range < DRY) {
//  PotValue = 0;
//  Serial.println("digital pot setting: Dry -> 5-darkest");
//}

// ****** Alternative mapping 2 ******
  switch (range) {
    case 0:    // DigPot = 22K (on my toaster, a bit more than 20K)
      PotValue = 54;
      Serial.println("digital pot setting: 5-darkest");
      break;
    case 1:    // DigPot = 15K
      PotValue = 41;
      Serial.println("digital pot setting: 4-dark");
      break;
    case 2:    // DigPot = 10K
      PotValue = 27;
      Serial.println("digital pot setting: 3-medium");
      break;
    case 3:    // DigPot = 5K
      PotValue = 13;
      Serial.println("digital pot setting: 2-light");
      break;
    case 4:    // DigPot = 0K
      PotValue = 0;
      Serial.println("digital pot setting: 1-Very Light");
      break;
  }
  
// PotValue = 128;         // test pot write function
digitalPotWrite(PotValue); // call function to set wiper value

// The following code is to read the toaster pot setting and
// adjust the speed of the pump accordingly.  Dry soil -> fastest
// pump speed
int ToasterPot = analogRead(ToasterPin); // Read 20K pot in toaster
Serial.print("Analog toaster pot value: ");
Serial.println(ToasterPot);

int pumpSpeed = map(ToasterPot, ToasterMin, ToasterMax, 4, 0); // Map toaster pot setting to 5 pump speeds
Serial.print("pumpSpeed: ");
Serial.println(pumpSpeed);

  switch (pumpSpeed) {
    case 0:    // Toaster Pot = 20K
      servo.writeMicroseconds(FASTEST);
      Serial.println("Toaster's Analog Pot at 5; Pump Fastest");
      break;    
    case 1:    // Toaster Pot = 15K
      servo.writeMicroseconds(FASTER);
      Serial.println("Toaster's Analog Pot at 4; Pump Faster");
      break;
    case 2:    // Toaster Pot = 10K
      servo.writeMicroseconds(FAST);
      Serial.println("Toaster's Analog Pot at 3; Pump Fast");
      break;
    case 3:    // Toaster Pot = 5K
      servo.writeMicroseconds(MED);
      Serial.println("Toaster's Analog Pot at 2; Pump Med");
      break;
    case 4:    // Toaster Pot = 0K
      servo.writeMicroseconds(SLOW);
      Serial.println("Toaster's Analog Pot at 1; Pump Slow");
      break;
  }
 delay (1000); // Delay for motor controller
}

int digitalPotWrite(int value) // Access the digitial pot
{
digitalWrite(CS, LOW); //select 50K digital pot chip
SPI.transfer(address); //select wiper
SPI.transfer(value); // assign value to wiper
digitalWrite(CS, HIGH); // deselect 50K digital pot chip
}
